from django.apps import AppConfig
from django.utils.translation import gettext_lazy


class JwtEmailAuthRotationConfig(AppConfig):
    name = "jwt_email_auth.rotation"
    verbose_name = gettext_lazy("JWT rotation")
    default_auto_field = "django.db.models.BigAutoField"



from django.utils.translation import gettext_lazy
from rest_framework import status
from rest_framework.exceptions import APIException

from .typing import Any, Optional


__all__ = [
    "ClaimNotUpdateable",
    "CorruptedDataException",
    "SendCodeCooldown",
    "ServerException",
    "UnexpectedClaim",
    "UserBanned",
]


class ServerException(APIException):
    status_code = status.HTTP_503_SERVICE_UNAVAILABLE
    default_detail = gettext_lazy("Server did not respond.")
    default_code = "server_down"


class SendCodeCooldown(APIException):
    status_code = status.HTTP_412_PRECONDITION_FAILED
    default_detail = gettext_lazy("This user is not allowed to send another login code yet.")
    default_code = "send_code_cooldown"


class UserBanned(APIException):
    status_code = status.HTTP_412_PRECONDITION_FAILED
    default_detail = gettext_lazy("Maximum number of attempts reached. Try again in %(x)s minutes.")
    default_code = "user_banned"

    def __init__(self, cooldown: int, detail: Optional[str] = None, code: Optional[str] = None):
        self.default_detail %= {"x": cooldown}
        super().__init__(detail, code)


class UnexpectedClaim(APIException):
    status_code = status.HTTP_412_PRECONDITION_FAILED
    default_detail = gettext_lazy("'%(claim)s' not found from the list of expected claims.")
    default_code = "unexpected_claim"

    def __init__(self, claim: Any, detail: Optional[str] = None, code: Optional[str] = None):
        self.default_detail %= {"claim": str(claim)}
        super().__init__(detail, code)


class ClaimNotUpdateable(UnexpectedClaim):
    status_code = status.HTTP_412_PRECONDITION_FAILED
    default_detail = gettext_lazy("Not allowed to update claim '%(claim)s'.")
    default_code = "claim_not_updateable"

    def __init__(self, claim: Any, detail: Optional[str] = None, code: Optional[str] = None):
        self.default_detail %= {"claim": str(claim)}
        super().__init__(detail, code)


class CorruptedDataException(APIException):
    status_code = status.HTTP_410_GONE
    default_detail = gettext_lazy("Data was corrupted.")
    default_code = "data_corruption"



import logging

from django.conf import settings
from django.utils.translation import gettext_lazy
from rest_framework.exceptions import AuthenticationFailed, NotAuthenticated
from rest_framework.permissions import BasePermission

from .settings import auth_settings
from .tokens import AccessToken


__all__ = [
    "HasValidJWT",
]


logger = logging.getLogger(__name__)


class HasValidJWT(BasePermission):

    message = gettext_lazy("Invalid token.")
    code = "permission_denied"

    def has_permission(self, request, view):
        if request.method == "OPTIONS" and settings.DEBUG and auth_settings.OPTIONS_SCHEMA_ACCESS:
            logger.debug("Allow access for OPTIONS requests in DEBUG mode.")
            return True

        try:
            AccessToken.from_request(request)
        except (AuthenticationFailed, NotAuthenticated) as error:
            logger.debug(error)
            self.message = error.detail
            self.code = error.detail.code
            return False

        return True



import logging
from datetime import timedelta
from inspect import cleandoc
from pathlib import Path

from django.test.signals import setting_changed
from settings_holder import SettingsHolder, reload_settings

from .typing import Any, Dict, List, Literal, NamedTuple, Optional, Set, Union


__all__ = [
    "auth_settings",
]


logger = logging.getLogger(__name__)


class JWTEmailAuthSettings(NamedTuple):
    # Whether emails should be sent or not. When off,
    # login code is logged instead (for development).
    SENDING_ON: bool = False
    #
    # When True, any code will work in login
    SKIP_CODE_CHECKS: bool = False
    #
    # List of emails for which code checks and email sending are off,
    # even if SKIP_CODE_CHECKS=False and/or SENDING_ON=True.
    # Useful for creating review accounts in an otherwise closed system.
    SKIP_CODE_CHECKS_FOR: List[str] = []
    #
    # How long an access token is valid for
    ACCESS_TOKEN_LIFETIME: timedelta = timedelta(minutes=5)
    #
    # How long a refresh token is valid for
    REFRESH_TOKEN_LIFETIME: timedelta = timedelta(days=14)
    #
    # How long a login code is stored in cache
    LOGIN_CODE_LIFETIME: timedelta = timedelta(minutes=5)
    #
    # After user has exceeded defined number of login attempts,
    # this is the cooldown until they can attempt login again.
    LOGIN_COOLDOWN: timedelta = timedelta(minutes=5)
    #
    # After a user has sent a login code, this is the cooldown until
    # they can send one again.
    CODE_SEND_COOLDOWN: timedelta = timedelta(minutes=1)
    #
    # How long after the creation of the JWT does it become valid.
    NOT_BEFORE_TIME: Optional[timedelta] = None
    #
    # If True, return a new refresh token when requesting a new
    # access token from RefreshTokenView. The old refresh token will
    # be invalid after the new one is created.
    ROTATE_REFRESH_TOKENS: bool = False
    #
    # Number of login attempts until user is banned
    LOGIN_ATTEMPTS: int = 10
    #
    # List of expected custom JWT claims
    EXPECTED_CLAIMS: List[str] = []
    #
    # Which expected claims can be updated without re-authentication
    UPDATEABLE_CLAIMS: List[str] = []
    #
    # Email sender. Default is settings.DEFAULT_FROM_EMAIL
    LOGIN_SENDING_EMAIL: Optional[str] = None
    #
    # Email subject line
    LOGIN_SUBJECT_LINE: str = "Login to Django"
    #
    # Message to send in email. Must have {code} and {valid}!
    LOGIN_EMAIL_MESSAGE: str = cleandoc(
        """
            Your login code:

            {code}

            This code is valid for the next {valid} minutes.
        """
    )
    #
    # Path to html_message template. Context must have {{ code }} and {{ valid }}!
    LOGIN_EMAIL_HTML_TEMPLATE: Optional[Path] = None
    #
    # Issuer of the JWT
    ISSUER: Optional[str] = None
    #
    # Intended recipient of the JWT
    AUDIENCE: Optional[str] = None
    #
    # A time margin in seconds for the expiration check
    LEEWAY: int = 0
    #
    # Algorithm to sign and decrypt the token with
    ALGORITHM: str = "EdDSA"
    #
    # Authorization scheme used in Authorization header, as in `HEADER_PREFIX token`
    HEADER_PREFIX: str = "Bearer"
    #
    # Additional JWT header fields
    EXTRA_HEADERS: Optional[Dict[str, str]] = None
    #
    # Cache prefix
    CACHE_PREFIX: str = "Django"
    #
    # When True (default), OPTIONS requests can be made to the endpoint without token for schema access
    OPTIONS_SCHEMA_ACCESS: bool = True
    #
    # Function to load JWT signing key.
    # Takes no arguments. Returns the Ed25519PrivateKey-object used to check the JWT signature.
    # Default function loads an example key, DO NOT USE IT IN PRODUCTION!
    SIGNING_KEY: str = "jwt_email_auth.utils.load_example_signing_key"
    #
    # If set, JWT will be encrypted with AES in GCM-mode using this as the secret key.
    # Should be either 16, 24, or 32 bytes, encoded to base64, e.g., `b64encode(urandom(32)).decode()`
    CIPHER_KEY: Optional[str] = None
    #
    # Function to generate a login code.
    # Takes no arguments. Returns a login code (str).
    CODE_GENERATOR: str = "jwt_email_auth.utils.random_code"
    #
    # Function that sends the login email.
    # Arguments: email (str), and login data (Dict[str, Any]), and request (Request). Returns None.
    SEND_LOGIN_CODE_CALLBACK: str = "jwt_email_auth.utils.send_login_email"
    #
    # Function to use for validating user and providing login data.
    # Arguments: email (str). Returns login data (Dict[str, Any]).
    LOGIN_VALIDATION_AND_DATA_CALLBACK: str = "jwt_email_auth.utils.validate_login_and_provide_login_data"
    #
    # Function to generate cache key for storing user's login attempts.
    # Arguments: request (Request). Returns a cache key (str).
    LOGIN_BLOCKER_CACHE_KEY_CALLBACK: str = "jwt_email_auth.utils.blocking_cache_key_from_ip"
    #
    # Function for additional handling for blocked users.
    # Arguments: request (Request). Returns None.
    USER_BLOCKED_ADDITIONAL_HANDLER: str = "jwt_email_auth.utils.blocking_handler"
    #
    # Function to check if token user still exists in refresh view.
    # Arguments: refresh (RefreshToken). Returns None or raises `rest_framework.exceptions.NotFound`.
    USER_CHECK_CALLBACK: str = "jwt_email_auth.utils.user_check_callback"
    #
    # If True, can authenticate with tokens in Authorization header.
    # Set this to False and `USE_COOKIES` to True to only allow cookie authentication.
    USE_TOKENS: bool = True
    #
    # If True, can authenticate with tokens in HttpOnly headers.
    # Cookies will be checked before Authorization headers if they are enabled.
    USE_COOKIES: bool = False
    #
    # Default login method to use if none is given in Prefer-headers. If not set,
    # cookie-based login will be used if enabled, else token-based.
    DEFAULT_LOGIN_METHOD: Optional[Literal["token", "cookies"]] = None  # noqa: F821
    #
    # Cookie key to use for the access token
    ACCESS_TOKEN_KEY: str = "access"
    #
    # Cookie key to use for the refresh token
    REFRESH_TOKEN_KEY: str = "refresh"
    #
    # Indicates that the cookie is sent to the server only when
    # a request is made with the https: scheme (except on localhost),
    # and therefore, is more resistant to man-in-the-middle attacks.
    SET_COOKIE_SECURE: bool = True
    #
    # Indicates the path that must exist in the requested URL
    # for the browser to send the access token cookie.
    SET_COOKIE_ACCESS_PATH: str = "/"
    #
    # Indicates the path that must exist in the requested URL
    # for the browser to send the refresh token cookie.
    SET_COOKIE_REFRESH_PATH: str = "/"
    #
    # Defines the host to which the cookie will be sent.
    # If None, this attribute defaults to the host of the
    # current document URL, not including subdomains.
    SET_COOKIE_DOMAIN: Optional[str] = None
    #
    # If True, forbids JavaScript from accessing the cookie.
    SET_COOKIE_HTTPONLY: bool = True
    #
    # Controls whether a cookie is sent with cross-origin requests,
    # providing some protection against cross-site request forgery attacks (CSRF).
    SET_COOKIE_SAMESITE: Literal["lax", "strict", "none"] = "lax"  # noqa: F821
    #
    # IP address spoofing prevention settings:
    # https://github.com/un33k/django-ipware/blob/master/README.md#advanced-users
    #
    # Indicate whether the originating client is on the right or left in the X-Forwarded-For header
    PROXY_ORDER: Literal["left-most", "right-most"] = "left-most"  # noqa: F821
    #
    # Number of proxies between the server and internet
    PROXY_COUNT: Optional[int] = None
    #
    # Only these proxy IPs are allowed connections
    PROXY_TRUSTED_IPS: Optional[List[str]] = None
    #
    # Meta precedence order
    REQUEST_HEADER_ORDER: Optional[List[str]] = None


SETTING_NAME: str = "JWT_EMAIL_AUTH"

DEFAULTS: Dict[str, Any] = JWTEmailAuthSettings()._asdict()

IMPORT_STRINGS: Set[Union[bytes, str]] = {
    b"SIGNING_KEY",
    "CODE_GENERATOR",
    "SEND_LOGIN_CODE_CALLBACK",
    "LOGIN_VALIDATION_AND_DATA_CALLBACK",
    "LOGIN_BLOCKER_CACHE_KEY_CALLBACK",
    "USER_BLOCKED_ADDITIONAL_HANDLER",
    "USER_CHECK_CALLBACK",
}

REMOVED_SETTINGS: Set[str] = {
    "LOGIN_EMAIL_CALLBACK",
    "SEND_EMAILS",
    "LOGIN_DATA",
    "VALIDATION_CALLBACK",
    "LOGIN_CALLBACK",
    "BLOCKING_HANDLER",
    "LOGIN_BLOCKER_CALLBACK",
    "REFRESH_VIEW_BOTH_TOKENS",
    "SET_COOKIE_PATH",
}


auth_settings = SettingsHolder(
    setting_name=SETTING_NAME,
    defaults=DEFAULTS,
    import_strings=IMPORT_STRINGS,
    removed_settings=REMOVED_SETTINGS,
)

reload_jwt_auth_settings = reload_settings(SETTING_NAME, auth_settings)

setting_changed.connect(reload_jwt_auth_settings)



from django.utils.functional import cached_property
from rest_framework.exceptions import AuthenticationFailed, NotAuthenticated, ValidationError
from rest_framework.fields import CharField, HiddenField, empty
from rest_framework.request import Request

from .tokens import AccessToken
from .typing import Any, Optional
from .utils import valid_jwt_format


__all__ = [
    "AutoTokenField",
    "TokenField",
]


class TokenField(CharField):
    """Validates incoming tokens are in the correct format."""

    default_validators = [valid_jwt_format]


class AutoTokenField(HiddenField):
    """A field that automatically populates from the parent serializers context.
    The context dictionary's must include a Request object, and that Request object
    must have the authorization token, either in its cookies or the 'Authorization' header
    depending on settings, from which the JWT is created. If any of these are missing,
    or the created JWT is not valid, an error will be raised.

    Parent serializers 'validated_data' attribute will contain an 'AccessToken' object.
    Parent serializers 'data' attribute will contain the encoded token as a string.

    AutoTokenField is not shown in the generated schema (due to sublassing HiddenField),
    since the purpose for this field is to accept the token from the request and not
    from user input.
    """

    def __init__(self, **kwargs):
        kwargs["default"] = empty
        super().__init__(**kwargs)

        # Both must always be False so that token appears
        # in both 'validated_data' and 'data' attributes
        self.write_only = False
        self.read_only = False

    @cached_property
    def _default(self) -> AccessToken:  # type: ignore
        request: Optional[Request] = self.parent.context.get("request")
        if request is None or not isinstance(request, Request):
            raise ValidationError("Must include a Request object in the context of the Serializer.")

        try:
            return AccessToken.from_request(request)
        except (AuthenticationFailed, NotAuthenticated) as error:
            raise ValidationError(error.detail) from error

    def get_default(self) -> AccessToken:  # type: ignore
        return self._default

    def run_validation(self, data: Any = ...) -> AccessToken:
        return self.get_default()

    def get_attribute(self, instance) -> AccessToken:
        return self.get_default()

    def to_representation(self, value: AccessToken) -> str:
        return str(value)



from typing import TYPE_CHECKING, Any, Dict, List, Literal, NamedTuple, Optional, Set, Tuple, Type, Union

from django.db.models import TextChoices


__all__ = [
    "Any",
    "Dict",
    "List",
    "Literal",
    "LoginMethod",
    "NamedTuple",
    "Optional",
    "Set",
    "Tuple",
    "Type",
    "TYPE_CHECKING",
    "Union",
]


class LoginMethod(TextChoices):
    TOKEN = "token"
    COOKIES = "cookies"



import logging

from django.core.cache import cache
from django.core.exceptions import ImproperlyConfigured
from django.http.request import HttpHeaders
from django.utils.translation import gettext_lazy
from rest_framework import status
from rest_framework.authentication import BaseAuthentication
from rest_framework.exceptions import APIException, AuthenticationFailed, NotFound, ParseError
from rest_framework.permissions import BasePermission
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.serializers import BaseSerializer
from rest_framework.views import APIView

from .authentication import JWTAuthentication
from .exceptions import (
    ClaimNotUpdateable,
    CorruptedDataException,
    SendCodeCooldown,
    ServerException,
    UnexpectedClaim,
    UserBanned,
)
from .permissions import HasValidJWT
from .schema import (
    LoginViewSchema,
    LogoutViewSchema,
    RefreshTokenViewSchema,
    SendLoginCodeViewSchema,
    TokenClaimViewSchema,
    UpdateTokenViewSchema,
)
from .serializers import (
    LoginSerializer,
    LogoutSerializer,
    RefreshTokenSerializer,
    SendLoginCodeSerializer,
    TokenClaimSerializer,
    TokenUpdateSerializer,
)
from .settings import auth_settings
from .tokens import AccessToken, RefreshToken, TokenType
from .typing import Any, Dict, List, LoginMethod, Tuple, Type
from .utils import (
    generate_code_sent_cache_key,
    generate_login_data_cache_key,
    get_id_value_from_request_data,
    user_is_blocked,
)


__all__ = [
    "LoginView",
    "LogoutView",
    "RefreshTokenView",
    "SendLoginCodeView",
    "TokenClaimView",
    "UpdateTokenView",
]


logger = logging.getLogger(__name__)


class BaseAuthView(APIView):
    """Base class for JWT authentication"""

    serializer_class: Type[BaseSerializer]

    def get_serializer(self, *args: Any, **kwargs: Any) -> BaseSerializer:
        kwargs["serializer_class"] = self.get_serializer_class()
        return self.initialize_serializer(*args, **kwargs)

    def initialize_serializer(self, *args: Any, **kwargs: Any) -> BaseSerializer:
        serializer_class: Type[BaseSerializer] = kwargs.pop("serializer_class")
        kwargs.setdefault("context", self.get_serializer_context())
        kwargs.setdefault("many", getattr(serializer_class, "many", False))
        return serializer_class(*args, **kwargs)

    def get_serializer_class(self) -> Type[BaseSerializer]:
        assert self.serializer_class, "Serializer class not defined"  # noqa: S101
        return self.serializer_class

    def get_serializer_context(self):
        return {"request": self.request, "format": self.format_kwarg, "view": self}

    @classmethod
    def get_extra_actions(cls) -> List[str]:
        return []

    def make_response(self, method: str, access: AccessToken, refresh: RefreshToken) -> Response:
        if method == LoginMethod.COOKIES and auth_settings.USE_COOKIES:
            return self.response_with_cookies(access, refresh)
        if method == LoginMethod.TOKEN and auth_settings.USE_TOKENS:
            return self.response_with_data(access, refresh)

        raise ImproperlyConfigured(f"Method {method!r} is not available.")  # pragma: no cover

    @staticmethod
    def response_with_data(access: AccessToken, refresh: RefreshToken) -> Response:
        data = {"access": str(access), "refresh": str(refresh)}
        return Response(data=data, status=status.HTTP_200_OK)

    @staticmethod
    def response_with_cookies(access: AccessToken, refresh: RefreshToken) -> Response:
        response = Response(status=status.HTTP_204_NO_CONTENT)
        response.set_cookie(
            key=TokenType.access,
            value=str(access),
            expires=access["exp"],
            path=auth_settings.SET_COOKIE_ACCESS_PATH,
            domain=auth_settings.SET_COOKIE_DOMAIN,
            secure=auth_settings.SET_COOKIE_SECURE,
            httponly=auth_settings.SET_COOKIE_HTTPONLY,
            samesite=auth_settings.SET_COOKIE_SAMESITE,
        )
        response.set_cookie(
            key=TokenType.refresh,
            value=str(refresh),
            expires=refresh["exp"],
            path=auth_settings.SET_COOKIE_REFRESH_PATH,
            domain=auth_settings.SET_COOKIE_DOMAIN,
            secure=auth_settings.SET_COOKIE_SECURE,
            httponly=auth_settings.SET_COOKIE_HTTPONLY,
            samesite=auth_settings.SET_COOKIE_SAMESITE,
        )
        return response

    @staticmethod
    def _get_refresh_token(cookies: Dict[str, str], data: Dict[str, Any], headers: HttpHeaders) -> Tuple[str, str]:
        token_from_cookies = cookies.get(TokenType.refresh)
        token_from_data = data.get("token")
        prefer = headers.get("Prefer")

        if prefer == "token" and auth_settings.USE_TOKENS and token_from_data is not None:
            return LoginMethod.TOKEN, token_from_data

        if auth_settings.USE_COOKIES and token_from_cookies is not None:
            return LoginMethod.COOKIES, token_from_cookies

        if auth_settings.USE_TOKENS and token_from_data is not None:
            return LoginMethod.TOKEN, token_from_data

        msg = "Could not find refresh token."

        if auth_settings.USE_COOKIES and not auth_settings.USE_TOKENS:
            raise APIException(f"{msg} Only cookie authentication is available.")

        if auth_settings.USE_TOKENS and not auth_settings.USE_COOKIES:
            raise APIException(f"{msg} Only token authentication is available.")

        raise APIException(f"{msg} Neither token or cookie authentication configured.")


class SendLoginCodeView(BaseAuthView):
    """Send a new login code."""

    serializer_class: Type[BaseSerializer] = SendLoginCodeSerializer

    authentication_classes: List[Type[BaseAuthentication]] = []
    permission_classes: List[Type[BasePermission]] = []

    schema = SendLoginCodeViewSchema()

    def post(self, request: Request, *args, **kwargs) -> Response:
        login_info = self.serializer_class(data=request.data)
        login_info.is_valid(raise_exception=True)
        data = login_info.data
        value = self._get_id_value(data)

        if user_is_blocked(request, record_attempt=False):
            raise SendCodeCooldown()

        login_data_cache_key = generate_login_data_cache_key(value)
        code_sent_cache_key = generate_code_sent_cache_key(value)

        login_data = self._get_login_data(code_sent_cache_key, login_data_cache_key, value)

        login_data["code"] = auth_settings.CODE_GENERATOR()
        logger.debug(login_data)
        cache.set(login_data_cache_key, login_data, auth_settings.LOGIN_CODE_LIFETIME.total_seconds())

        if not auth_settings.SENDING_ON or value in auth_settings.SKIP_CODE_CHECKS_FOR:
            logger.info(f"Login code: '{login_data['code']}'")
            return Response(status=status.HTTP_204_NO_CONTENT)

        try:
            auth_settings.SEND_LOGIN_CODE_CALLBACK(value, login_data, self.request)
            cache.set(code_sent_cache_key, 1, auth_settings.CODE_SEND_COOLDOWN.total_seconds())

        except Exception as error:
            cache.delete(login_data_cache_key)
            logger.critical(f"Login code sending failed: {type(error).__name__}('{error}')")
            raise ServerException(gettext_lazy("Failed to send login codes. Try again later.")) from error

        return Response(status=status.HTTP_204_NO_CONTENT)

    def _get_id_value(self, data: Dict[str, Any]) -> Any:
        return get_id_value_from_request_data(data)

    def _get_login_data(self, code_sent_cache_key: str, login_data_cache_key: str, value: Any) -> Dict[str, Any]:
        login_data = cache.get(login_data_cache_key)
        if login_data is None:
            login_data = auth_settings.LOGIN_VALIDATION_AND_DATA_CALLBACK(value)
        else:
            code_sent = cache.get(code_sent_cache_key)
            if code_sent is not None:
                raise SendCodeCooldown()
        return login_data


class LoginView(BaseAuthView):
    """Get new refresh and access token pair. Use Prefer-header to set the login method."""

    serializer_class: Type[BaseSerializer] = LoginSerializer

    authentication_classes: List[Type[BaseAuthentication]] = []
    permission_classes: List[Type[BasePermission]] = []

    schema = LoginViewSchema()

    def post(self, request: Request, *args, **kwargs) -> Response:
        login = self.serializer_class(data=request.data)
        login.is_valid(raise_exception=True)

        method = request.headers.get(
            "Prefer",
            auth_settings.DEFAULT_LOGIN_METHOD
            or (LoginMethod.COOKIES.value if auth_settings.USE_COOKIES else LoginMethod.TOKEN.value),
        )

        if method not in LoginMethod.values:
            raise ParseError(f"{method!r} not a valid login method. Use one of these: {LoginMethod.values!r}")
        if method == LoginMethod.COOKIES.value and not auth_settings.USE_COOKIES:
            raise ParseError("Cookie-based authentication not configured.")
        if method == LoginMethod.TOKEN.value and not auth_settings.USE_TOKENS:
            raise ParseError("Token-based authentication not configured.")

        data = login.data
        value = self._get_id_value(data)

        if user_is_blocked(request):
            raise UserBanned(cooldown=int(auth_settings.LOGIN_COOLDOWN.total_seconds() // 60))

        login_data_cache_key = generate_login_data_cache_key(value)
        code_sent_cache_key = generate_code_sent_cache_key(value)

        login_data = cache.get(login_data_cache_key)
        if login_data is None:
            raise NotFound(gettext_lazy("No login code found for '%(value)s'.") % {"value": value})

        login_code = login_data.pop("code", None)
        if not auth_settings.SKIP_CODE_CHECKS and value not in auth_settings.SKIP_CODE_CHECKS_FOR:
            if login_code != data["code"]:
                raise AuthenticationFailed(gettext_lazy("Incorrect login code."))

        cache.delete(code_sent_cache_key)
        cache.delete(login_data_cache_key)

        claim_data = self._get_claims(login_data)

        refresh = RefreshToken()
        if auth_settings.ROTATE_REFRESH_TOKENS:
            refresh.create_log()

        refresh.update(claim_data)
        access = refresh.new_access_token(sync=True)

        return self.make_response(method, access, refresh)

    def _get_id_value(self, data: Dict[str, Any]) -> Any:
        return get_id_value_from_request_data(data)

    def _get_claims(self, login_data: Dict[str, Any]) -> Dict[str, Any]:
        try:
            return {key: login_data[key] for key in auth_settings.EXPECTED_CLAIMS}
        except KeyError as error:
            raise CorruptedDataException(gettext_lazy("Data was corrupted. Try to send another login code.")) from error


class RefreshTokenView(BaseAuthView):
    """Get new access token from a refresh token."""

    serializer_class: Type[BaseSerializer] = RefreshTokenSerializer

    authentication_classes: List[Type[BaseAuthentication]] = []
    permission_classes: List[Type[BasePermission]] = []

    schema = RefreshTokenViewSchema()

    def post(self, request: Request, *args, **kwargs) -> Response:
        token = self.serializer_class(data=request.data)
        token.is_valid(raise_exception=True)
        data = token.data

        method, token_string = self._get_refresh_token(request.COOKIES, data, request.headers)
        refresh = RefreshToken(token=token_string)

        user_check = data.get("user_check", False)
        if user_check:
            auth_settings.USER_CHECK_CALLBACK(refresh)

        if auth_settings.ROTATE_REFRESH_TOKENS:
            refresh = refresh.rotate()

        access = refresh.new_access_token(sync=auth_settings.ROTATE_REFRESH_TOKENS)

        return self.make_response(method, access, refresh)


class LogoutView(BaseAuthView):
    """Invalidate refresh token when logging out."""

    serializer_class: Type[BaseSerializer] = LogoutSerializer

    authentication_classes: List[Type[BaseAuthentication]] = []
    permission_classes: List[Type[BasePermission]] = []

    schema = LogoutViewSchema()

    def post(self, request: Request, *args, **kwargs) -> Response:
        # Import is here so that jwt rotation remains optional
        from .rotation.models import RefreshTokenRotationLog

        token = self.serializer_class(data=request.data)
        token.is_valid(raise_exception=True)
        data = token.data
        _, token_string = self._get_refresh_token(request.COOKIES, data, request.headers)
        RefreshTokenRotationLog.objects.remove_by_token_title(token=token_string)
        return Response(status=status.HTTP_204_NO_CONTENT)


class UpdateTokenView(BaseAuthView):
    """Update token claims. Changes the token signatures."""

    serializer_class: Type[BaseSerializer] = TokenUpdateSerializer

    authentication_classes: List[Type[BaseAuthentication]] = []
    permission_classes: List[Type[BasePermission]] = []

    schema = UpdateTokenViewSchema()

    def post(self, request: Request, *args, **kwargs) -> Response:
        token = self.serializer_class(data=request.data)
        token.is_valid(raise_exception=True)
        data = token.data

        for claim in data["data"]:
            if claim not in auth_settings.EXPECTED_CLAIMS:
                raise UnexpectedClaim(claim=claim)
            if claim not in auth_settings.UPDATEABLE_CLAIMS:
                raise ClaimNotUpdateable(claim=claim)

        method, token_string = self._get_refresh_token(request.COOKIES, data, request.headers)
        refresh = RefreshToken(token=token_string)
        refresh.update(data["data"])
        if auth_settings.ROTATE_REFRESH_TOKENS:
            refresh = refresh.rotate()

        access = refresh.new_access_token(sync=auth_settings.ROTATE_REFRESH_TOKENS)

        return self.make_response(method, access, refresh)


class TokenClaimView(BaseAuthView):
    """Extract token claims."""

    serializer_class: Type[BaseSerializer] = TokenClaimSerializer

    authentication_classes: List[Type[BaseAuthentication]] = [JWTAuthentication]
    permission_classes: List[Type[BasePermission]] = [HasValidJWT]

    schema = TokenClaimViewSchema()

    def get(self, request: Request, *args, **kwargs) -> Response:
        access = AccessToken.from_request(request)
        return Response(data=access.payload, status=status.HTTP_200_OK)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['jwt_email_auth',
 'jwt_email_auth.rotation',
 'jwt_email_auth.rotation.migrations']

package_data = \
{'': ['*'], 'jwt_email_auth': ['locale/fi/LC_MESSAGES/*']}

install_requires = \
['Django>=3.2',
 'PyJWT>=2.3.0',
 'cffi>=1.15.0',
 'cryptography>=36.0.0',
 'django-ipware>=4.0.0',
 'django-settings-holder>=0.1.0',
 'djangorestframework>=3.12.0',
 'magic-specs>=0.0.4']

setup_kwargs = {
    'name': 'jwt-email-auth',
    'version': '0.11.4',
    'description': 'JWT authentication from email login codes.',
    'long_description': "# JSON Web Token Email Authentication\n\n[![Coverage Status][coverage-badge]][coverage]\n[![GitHub Workflow Status][status-badge]][status]\n[![PyPI][pypi-badge]][pypi]\n[![GitHub][licence-badge]][licence]\n[![GitHub Last Commit][repo-badge]][repo]\n[![GitHub Issues][issues-badge]][issues]\n[![Downloads][downloads-badge]][pypi]\n\n[![Python Version][version-badge]][pypi]\n[![Django Version][django-badge]][pypi]\n[![DRF Version][drf-badge]][pypi]\n\n```shell\npip install jwt-email-auth\n```\n\n---\n\n**Documentation**: [https://mrthearman.github.io/jwt-email-auth/](https://mrthearman.github.io/jwt-email-auth/)\n\n**Source Code**: [https://github.com/MrThearMan/jwt-email-auth/](https://github.com/MrThearMan/jwt-email-auth/)\n\n---\n\n\nThis module enables JSON Web Token Authentication in Django Rest framework without using Django's User model.\nInstead, login information is stored in [cache][cache], a login code is sent to the user's email inbox,\nand then the cached information is obtained using the code that was sent to the given email.\n\n\n[cache]: https://docs.djangoproject.com/en/3.2/topics/cache/#the-low-level-cache-api\n\n[coverage-badge]: https://coveralls.io/repos/github/MrThearMan/jwt-email-auth/badge.svg?branch=main\n[status-badge]: https://img.shields.io/github/workflow/status/MrThearMan/jwt-email-auth/Test\n[pypi-badge]: https://img.shields.io/pypi/v/jwt-email-auth\n[licence-badge]: https://img.shields.io/github/license/MrThearMan/jwt-email-auth\n[repo-badge]: https://img.shields.io/github/last-commit/MrThearMan/jwt-email-auth\n[issues-badge]: https://img.shields.io/github/issues-raw/MrThearMan/jwt-email-auth\n[version-badge]: https://img.shields.io/pypi/pyversions/jwt-email-auth\n[downloads-badge]: https://img.shields.io/pypi/dm/jwt-email-auth\n[django-badge]: https://img.shields.io/pypi/djversions/jwt-email-auth\n[drf-badge]: https://img.shields.io/badge/drf%20versions-3.12.0%20--%203.13.0-blue\n\n[coverage]: https://coveralls.io/github/MrThearMan/jwt-email-auth?branch=main\n[status]: https://github.com/MrThearMan/jwt-email-auth/actions/workflows/test.yml\n[pypi]: https://pypi.org/project/jwt-email-auth\n[licence]: https://github.com/MrThearMan/jwt-email-auth/blob/main/LICENSE\n[repo]: https://github.com/MrThearMan/jwt-email-auth/commits/main\n[issues]: https://github.com/MrThearMan/jwt-email-auth/issues\n",
    'author': 'Matti Lamppu',
    'author_email': 'lamppu.matti.akseli@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/MrThearMan/jwt-email-auth',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4',
}


setup(**setup_kwargs)
